package tempz.gui;


import static tempz.gui.TempzWindow.*;

import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;

import weather.*;

/**
 * The observer of all GUI components in the TempzWindow.
 * 
 * @author Prof. David Bernstein, James Madison University
 */
public class TempzController implements ActionListener, DocumentListener, ItemListener
{
  static final String DATA_FILES         = "Data Files";
  static final String ERROR              = "Error";
  static final String ERROR_OPENING_FILE = "Error Opening File";
  static final String INVALID_FILE_TYPE  = "Invalid File Type";
  static final String NO_MATCHES         = "No Matches";
  
  private FileNameExtensionFilter temperatureFilter;
  private TempzWindow window;
  private JFileChooser fileChooser;


  private TemperatureDatabase database;
  
  /**
   * Default Constructor.
   */
  public TempzController()
  {
    super();
    temperatureFilter = new FileNameExtensionFilter(DATA_FILES, "tem");
    fileChooser = new JFileChooser();
    fileChooser.setCurrentDirectory(new File("."));
    
    database = new TemperatureDatabase("United States");
  }
  
  /**
   * Handle actionPerformed() message (required by ActionListener).
   * 
   * @param ae   The ActionEvent that generated the message
   */
  public void actionPerformed(ActionEvent ae)
  {
    String ac = ae.getActionCommand();
    
    if (ac.equals(CALCULATE))   calculate();
    else if (ac.equals(EXIT))   System.exit(0);
    else if (ac.equals(READ))   read();
  }
  
  /**
   * Handle a CALCULATE request.
   */
  private void calculate()
  {
    List<String>  results = new ArrayList<String>();
    window.displayResults(null);
    
    TemperatureTable table = database.get(window.getLocality());
    if (table == null)
    {
      results.add("Invalid locality.");
    }
    else
    {
      int year = window.getYear();
      int month = window.getMonth();
      try
      {
        double max = table.getAverageMax(year, month);
        double min = table.getAverageMin(year, month);
        results.add(String.format("Average Maximum Temperature: %5.2f", max));
        results.add(String.format("Average Minimum Temperature: %5.2f", min));
      }
      catch (IllegalArgumentException iae)
      {
        results.add("Sorry, there is no information available for that date.");
      }
    }
    window.displayResults(results);

  }

  /**
   * Handle changedUpdate messages.
   * 
   * @param e The event that generated the message
   */
  @Override
  public void changedUpdate(DocumentEvent e)
  {
    // Shouldn't happen
  }

  /**
   * Determine if calculations are possible.
   */
  private void checkCalculatability()
  {
    boolean disable = window.localityField.getSelectedItem() == null
        || window.localityField.getSelectedItem().equals("")
        || window.monthField.getText().equals("") || window.monthField.getText() == null
        || window.yearField.getText().equals("") || window.yearField.getText() == null;
    
    window.enableCalculation(!disable);
  }



  /**
   * Handle insertUpdate messages.
   * 
   * @param de The event that generated the message
   */
  @Override
  public void insertUpdate(DocumentEvent de)
  {
    checkCalculatability();
  }
  
  /**
   * Handle itemStateChanged message.
   * 
   * @param ie The event that generated the message
   */
  @Override
  public void itemStateChanged(ItemEvent ie)
  {
    checkCalculatability();
  }
  
  /**
   * Handle a READ request.
   */
  private void read()
  {
    fileChooser.setFileFilter(temperatureFilter);
    int result = fileChooser.showOpenDialog(null);
    if (result == JFileChooser.APPROVE_OPTION)
    {
      String path = fileChooser.getSelectedFile().getPath();
      String extension = temperatureFilter.getExtensions()[0];
      if (path.endsWith(extension))
      {
        String name = path.substring(0, path.length() - 4);
        try
        {
          TemperatureTable table = new TemperatureTable();
          table.read(name);
          database.put(table.getLocality(), table);
          
          setTemperatureTableIsAvailable(true);
          window.addLocality(table.getLocality());
        }
        catch (IOException ioe)
        {
          ioe.printStackTrace();
          JOptionPane.showMessageDialog(null, ERROR_OPENING_FILE, ERROR, JOptionPane.ERROR_MESSAGE);
        }
      }
      else
      {
        JOptionPane.showMessageDialog(null, INVALID_FILE_TYPE, ERROR, JOptionPane.ERROR_MESSAGE);
      }
    }
  }

  /**
   * Handle removeUpdate messages.
   * 
   * @param de The event that generated the message
   */
  @Override
  public void removeUpdate(DocumentEvent e)
  {
    checkCalculatability();
  } 
  
  /**
   * Set whether a TemperatureTable is available to use or not.
   * 
   * @param boolean true to indicate a TemperatureTable is available; false otherwise
   */
  private void setTemperatureTableIsAvailable(boolean available)
  {
    window.enableDataEntry(available);
  }
  
  /**
   * Set the TempzWindow that this object is controlling.
   * 
   * @param window  The window.
   */
  public void setWindow(TempzWindow window)
  {
    this.window = window;
  }
}
