package tempz.gui;

import gui.*;
import java.awt.*;
//import java.net.*;
import java.util.Arrays;
import java.util.List;
import javax.swing.*;

/**
 * The main window in the Tempz application.
 * 
 * @author Prof. David Bernstein, James Madison University
 */
public class TempzWindow extends JFrame
{
  // Constants with package visibility
  static final String CALCULATE      = "Calculate";
  static final String FILE           = "File";
  static final String EXIT           = "Exit";
  static final String LOCATION       = "Locality";
  static final String MONTH          = "Month";
  static final String READ           = "Read";
  static final String YEAR           = "Year";
  static final String UTILITIES      = "Utilities";

  // Constants with private visibility
  private static final long serialVersionUID = 1L;

  // Attributes
  private JButton            calculateButton;
          JComboBox<String>  localityField;
  private JList<String>      resultsArea;
          JTextField         monthField, yearField;
  private String             fileName;
  private TempzController    controller;


  /**
   * Explicit Value Constructor.
   * 
   * @param controller  The observer for all GUI events.
   */
  public TempzWindow(TempzController controller)
  {
    super();
    this.controller = controller;
    controller.setWindow(this);
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    setIconImage(loadImage("Tempz_icon.png"));

    yearField      = createJTextField();
    yearField.setEnabled(false);
    yearField.getDocument().addDocumentListener(controller);
    
    monthField     = createJTextField();
    monthField.setEnabled(false);
    monthField.getDocument().addDocumentListener(controller);

    localityField  = createJComboBox(null);
    localityField.setEditable(false);
    localityField.setEnabled(false);
    localityField.addItemListener(controller);
    
    resultsArea = new JList<String>(new DefaultListModel<String>());
    calculateButton = createJButton("Calculate.png", CALCULATE);

    setFileName(null);
    
    performLayout();
    createMenuBar();

    enableCalculation(false);
    setFileName(null);
    setSize(800, 400);
  }

  /**
   * Add a "valid" locality.
   * 
   *  @param locality   The "valid" locality to add (or null to reset)
   */
  public void addLocality(String locality)
  {
    addTo(localityField, locality);
  }
  
  /**
   * Add an item to a JComboBox (if it isn't already there).
   * 
   * @param comboBox The JComboBox to add to
   * @param item     The item to add
   */
  private void addTo(JComboBox<String> comboBox, String item)
  {
    int n = comboBox.getItemCount();
    String[] items = new String[n];
    for (int i=1; i<n; i++) // Don't add the empty item
    {
      items[i-1] = comboBox.getItemAt(i);
      if (items[i-1].contentEquals(item)) return; // Don't duplicate
    }
    items[n-1] = item;
    
    populate(comboBox, items);
  }
  
  /**
   * Create a JPanel containing a JComponent
   * and a titled-border.
   *
   * @param title      The title
   * @param component  The JComponent
   * @return The JPanel
   */
  private JPanel createTitledComponent(String title, JComponent component)
  {
    JPanel result = new JPanel();
    result.setLayout(new BorderLayout());
    result.add(component, BorderLayout.CENTER);
    result.setBorder(BorderFactory.createTitledBorder(title));
    
    return result;
  }

  /**
   * Create a JButton.
   * 
   * @param name          The name of the Image
   * @param actionCommand The action command
   * @return The JButton
   */
  private JButton createJButton(String name, String actionCommand)
  {
    JButton result = new JButton(loadImageIcon(name));
    result.setActionCommand(actionCommand);
    result.addActionListener(controller);
    return result;
  }

  /**
   * Create a JComboBox.
   * 
   * @param items  The items to include (after the blank)
   * @return The JComboBox
   */
  private JComboBox<String> createJComboBox(String[] items)
  {
    JComboBox<String> result = new JComboBox<String>();
    result.setEditable(true);
    populate(result, items);
 
    return result;
  }

  /**
   * Create a JTextField.
   * 
   * @return The JTextField
   */
  private JTextField createJTextField()
  {
    JTextField    result = new JTextField("", 6);
    return result;
  }

  /**
   * Create the JMenuBar.
   */
  private void createMenuBar()
  {
    JMenuBar  menuBar = new JMenuBar();
    JMenu     menu;
    JMenuItem item;

    menu = new JMenu(FILE);
    {
      item = new JMenuItem(EXIT);
      item.addActionListener(controller);
      menu.add(item);
    }
    menuBar.add(menu);
    
    menu = new JMenu(UTILITIES);
    {
      item = new JMenuItem(READ);
      item.addActionListener(controller);
      menu.add(item);
    }
    menuBar.add(menu);

    setJMenuBar(menuBar);
  }

  /**
   * Display result.
   * 
   * @param results The results to display
   */
  public void displayResults(List<String> results)
  {
    DefaultListModel<String> model = (DefaultListModel<String>)resultsArea.getModel();
    model.clear();
    if (results != null)
    {
      for (String line: results) model.addElement(line);
    }
  }

  /**
   * Enable/disable the CALCULATE action.
   * 
   * @param enable true to enable; false to disable
   */
  public void enableCalculation(boolean enable)
  {
    calculateButton.setEnabled(enable);
  }
  
  /**
   * Enable/disable data entry.
   * 
   * @param enable true to enable; false to disable
   */
  public void enableDataEntry(boolean enable)
  {
    monthField.setEnabled(enable);
    yearField.setEnabled(enable);
    localityField.setEnabled(enable);
  }
  
  /**
   * Get the file name for the current file.
   * 
   * @return  The file name
   */
  public String getFileName()
  {
    return fileName;
  }

  /**
   * Get an int from a JTextField
   * 
   * @param field The field
   * @return The int
   */
  public int getInt(JTextField field)
  {
    int    result;
    String text = field.getText();
    if (text != null)
    {
      try
      {
        result = Integer.parseInt(text);
      }
      catch (NumberFormatException nfe)
      {
        result = 0;
      }
    }
    else
    {
      result = 0;
    }
    
    return result;
  }

  /**
   * Get the location for the current user input.
   * 
   * @return  The location
   */
  public String getLocality()
  {
    return (String)localityField.getSelectedItem();
  }

  /**
   * Get the month.
   * 
   * @return The month
   */
  public int getMonth()
  {
	  return getInt(monthField);
  }

  /**
   * Get the year.
   * 
   * @return The year
   */
  public int getYear()
  {
	  return getInt(yearField);
  }

  /**
   * Load an ImageIcon.
   * 
   * @param name  The name of the file containing the ImageIcon
   * @return      The ImageIcon
   */
  private ImageIcon loadImageIcon(String name)
  {
	  ImageIcon icon = new ImageIcon(name);
	  return icon;
  }

  /**
   * Load an Image.
   * 
   * @param name  The name of the file containing the Image
   * @return      The Image
   */
  private Image loadImage(String name)
  {
    return loadImageIcon(name).getImage();
  }


  /**
   * Layout this Component.
   * 
   */
  private void performLayout()
  {
    JPanel contentPane = (JPanel)getContentPane();

    contentPane.setLayout(new BorderLayout());

    JToolBar toolBar = new JToolBar();
    toolBar.setFloatable(false);
    toolBar.add(calculateButton);
    contentPane.add(toolBar, BorderLayout.NORTH);

    Row entryPanel = new Row();
    entryPanel.add(createTitledComponent(YEAR, yearField),         1,     0.0);
    entryPanel.add(createTitledComponent(MONTH, monthField),       2,     0.0);
    entryPanel.add(createTitledComponent(LOCATION, localityField), 3,   100.0);

    JPanel center = new JPanel();
    center.setLayout(new BorderLayout());
    center.add(entryPanel, BorderLayout.NORTH);
    JScrollPane sp = new JScrollPane(resultsArea);
    sp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    center.add(sp,  BorderLayout.CENTER);

    contentPane.add(center, BorderLayout.CENTER);
  }
  
  /**
   * Populate a JComboBox.
   * 
   * @param comboBox The JComboBox to populate
   * @param items    The items to populate it with (or null to reset)
   */
  private void populate(JComboBox<String> comboBox, String[] items)
  {
    comboBox.removeAllItems();
    comboBox.addItem("");
    if (items != null)
    {
      Arrays.sort(items);
      for (String item: items) comboBox.addItem(item);
    }
  }
  
  /**
   * Set the name of the current StreetNetwork.
   * 
   * @param name  The name of the file containing the current StreetNetwork
   */
  public void setFileName(String name)
  {
    fileName = name;
    String title = "Tempz";
    if ((name != null) && !name.equals("")) title += " - " + name;
    setTitle(title);
  }
}
