package weather;

import java.io.*;
import java.util.*;

/**
 * A collection of TemperaturObservation objects for a particular locality.
 * 
 * @author Prof. David Bernstein, James Madison University
 */
public class TemperatureTable
{
  private List<TemperatureObservation> data;
  private String locality;
  
  /**
   * Default Constructor.
   */
  public TemperatureTable()
  {
    data = new ArrayList<TemperatureObservation>();
    locality = null;
  }
  
  /**
   * Get the average maximum temperature for a particular month.
   * 
   * @param year The year of interest
   * @param month The month of interest
   * @return The everage maximum temperature for the month
   * @throws IllegalArgumentException if there are no data 
   */
  public double getAverageMax(int year, int month) throws IllegalArgumentException
  {
    List<TemperatureObservation> list = getList(year, month);
    double total = 0.0;
    for (TemperatureObservation temp: list) total += temp.getMax();

    return total / list.size();
  }
  
  /**
   * Get the average minimum temperature for a particular month.
   * 
   * @param year The year of interest
   * @param month The month of interest
   * @return The average minimum temperature for the month
   * @throws IllegalArgumentException if there are no data 
   */
  public double getAverageMin(int year, int month) throws IllegalArgumentException
  {
    List<TemperatureObservation> list = getList(year, month);
    double total = 0.0;
    for (TemperatureObservation temp: list) total += temp.getMin();

    return total / list.size();
  }
  
  /**
   * Get the locality for this TemperatureTable;
   * 
   * @return The locality (or null if the TemperatureTable is empty)
   */
  public String getLocality()
  {
    return locality; 
  }
  
  /**
   * Get all of the TemperatureObservation objects for a particular month.
   * 
   * @param year The year of interest
   * @param month The month of interest
   * @return The List of TemperatureObservation objects
   * @throws IllegalArgumentException
   */
  public List<TemperatureObservation> getList(int year, int month) throws IllegalArgumentException
  {
    ArrayList<TemperatureObservation> result = new ArrayList<TemperatureObservation>();
    for (TemperatureObservation obs: data)
    {
      if ((obs.getYear() == year) && (obs.getMonth() == month)) result.add(obs);
    }
    if (result.size() == 0) throw new IllegalArgumentException("No such month.");
    return result;
  }
  
  /**
   * Read a file containing String representations of TemperatureObservation objects.
   * 
   * @param fileName The name of the file to read from (without an extension)
   * @throws IOException if there is a problem
   */
  public void read(String fileName) throws IOException
  {
    try (BufferedReader in = new BufferedReader(new FileReader(fileName + ".tem"));)
    {
      String line;
      while ((line = in.readLine()) != null)
      {
        try
        {
          TemperatureObservation temp = TemperatureObservation.parseTemperatureObservation(line);
          if (locality == null) locality = temp.getName();
          data.add(temp);
        }
        catch (IllegalArgumentException iae)
        {
          // Ignore
        }
      }
    }
  }
}
